import requests
from cnaas_sync.utils import get_number_of_pages

# from utils import get_number_of_pages


class NAV:
    def __init__(self, base_url, api_token, verbose=True):
        self.base_url = base_url
        self.api_token = api_token
        self.headers = {"Authorization": "Token " + self.api_token}
        self.verbose = verbose

    def _print(self, value):
        if self.verbose:
            print(value)

    def get_all_rooms(self):
        res = requests.get(self.base_url + "/api/1/room/", headers=self.headers)
        if not res.status_code == 200:
            print(f"Failed to fetch devices from NAV: {res.status_code}")
            return False
        self._print("NAV ROOMS:")
        for room in res.json()["results"]:
            self._print(f"Room {room['id']} found in NAV!")

        return res.json()["results"]

    def get_all_devices(self, page_size=100):
        res = requests.get(
            self.base_url + f"/api/1/netbox/?page=1&page_size={page_size}",
            headers=self.headers,
        )
        if not res.status_code == 200:
            print(f"Failed to fetch devices from NAV: {res.status_code}")
            return False
        data_json = res.json()
        total_count = int(data_json["count"])
        num_pages = get_number_of_pages(total_count, page_size)
        accumulated_res = data_json["results"]
        for page in range(2, num_pages + 1):
            res_tmp = requests.get(
                self.base_url + f"/api/1/netbox/?page={page}&page_size={page_size}",
                headers=self.headers,
            )
            if "results" not in res_tmp.json():
                print(
                    "Failed to fetch devices from NMS! Status Code: "
                    + str(res_tmp.status_code)
                )
                continue
            accumulated_res.extend(res_tmp.json()["results"])  # extend vs +=
        return accumulated_res

    def add_new_managed_device(
        self, hostname, switches, category_id, management_ip, room, device_type
    ):
        data = {
            "ip": management_ip[0],
            "roomid": room,
            "organizationid": "SUNET",
            "categoryid": category_id,
            "sysname": f"{hostname}",
            "profiles": [1],
            "data": {"DEVICE_TYPE": device_type},
        }
        for device in switches:
            if device["ip"] == management_ip[0]:
                self._print(f"Switch {device['sysname']} found in NAV!")
                return False
        self._print(f"Adding {hostname}")
        res2 = requests.post(
            self.base_url + "/api/1/netbox/",
            headers=self.headers,
            json=data,
        )
        if res2.status_code != 201:
            print(f"Could not create device!  {res2}")
            return False

    def update(self, ni_devices, nms):
        all_nav_devices = self.get_all_devices()
        self._print("**" * 20)
        for device in ni_devices:
            if device["node"]["operational_state"] != "In service":
                continue
            hostname = device["node_name"]
            device_type = nms.get_device_type(hostname)
            if device_type == "CORE" or device_type == "DIST":
                category_id = "GSW"
            elif device_type == "ACCESS":
                category_id = "SW"
            else:
                continue
            management_ip = device["node"]["ip_addresses"]
            self.add_new_managed_device(
                hostname,
                all_nav_devices,
                category_id,
                management_ip,
                "myroom",
                device_type,
            )
            self._print("**" * 20)
